@@ -18,8 +18,6 @@
 
 #include <usual/cfparser.h>
 
-#include <string.h>
-
 #ifdef HAVE_PWD_H
 #include <pwd.h>
 #endif
@@ -28,6 +26,9 @@
 #include <usual/fileutil.h>
 #include <usual/logging.h>
 #include <usual/time.h>
+#include <usual/string.h>
+
+#define MAX_INCLUDE 10
 
 /*
  * INI file parser.
@@ -43,7 +44,7 @@ static int count_lines(const char *s, const char *end)
 	return lineno;
 }
 
-bool parse_ini_file(const char *fn, cf_handler_f user_handler, void *arg)
+static bool parse_ini_file_internal(const char *fn, cf_handler_f user_handler, void *arg, int inclevel)
 {
 	char *buf;
 	char *p, *key, *val;
@@ -60,6 +61,38 @@ bool parse_ini_file(const char *fn, cf_handler_f user_handler, void *arg)
 		/* space at the start of line - including empty lines */
 		while (*p && isspace(*p)) p++;
 
+		if (strncmp(p, "%include", 8) == 0 && p[8] != 0 && isblank(p[8])) {
+			if (inclevel >= MAX_INCLUDE) {
+				log_error("include nesting level too deep (%s:%d), stopping loading",
+					  fn, count_lines(buf, p));
+				goto failed;
+			}
+			p += 8;
+			while (*p && isblank(*p)) p++;
+			/* now read value */
+			val = p;
+			while (*p && (*p != '\n'))
+				p++;
+			vlen = p - val;
+			/* eat space at end */
+			while (vlen > 0 && isspace(val[vlen - 1]))
+				vlen--;
+
+			/*
+			 * val now has the name of the file to be included.
+			 * Process it recursively.
+			 */
+			o1 = val[vlen];
+			val[vlen] = 0;
+			log_debug("processing include: %s", val);
+			ok = parse_ini_file_internal(val, user_handler, arg, inclevel + 1);
+			val[vlen] = o1;
+			if (!ok)
+				goto failed;
+			log_debug("returned to processing file %s", fn);
+			continue;
+		}
+
 		/* skip comment lines */
 		if (*p == '#' || *p == ';') {
 			while (*p && *p != '\n') p++;
@@ -141,6 +174,11 @@ failed:
 	return false;
 }
 
+bool parse_ini_file(const char *fn, cf_handler_f user_handler, void *arg)
+{
+	return parse_ini_file_internal(fn, user_handler, arg, 0);
+}
+
 /*
  * Config framework.
  */
@@ -278,6 +316,7 @@ struct LoaderCtx {
 	const struct CfContext *cf;
 	const char *cur_sect;
 	void *top_base;
+	bool got_main_sect;
 };
 
 static bool fill_defaults(struct LoaderCtx *ctx)
@@ -289,6 +328,9 @@ static bool fill_defaults(struct LoaderCtx *ctx)
 	if (!s)
 		goto fail;
 
+	if (s == ctx->cf->sect_list)
+		ctx->got_main_sect = true;
+
 	if (s->section_start) {
 		if (!s->section_start(ctx->top_base, ctx->cur_sect))
 			return false;
@@ -332,14 +374,18 @@ static bool load_handler(void *arg, bool is_sect, const char *key, const char *v
 
 bool cf_load_file(const struct CfContext *cf, const char *fn)
 {
-	struct LoaderCtx ctx = {
-		.cf = cf,
-		.cur_sect = NULL,
-	};
+	struct LoaderCtx ctx;
+	bool ok;
+	memset(&ctx, 0, sizeof(ctx));
+	ctx.cf = cf;
 
-	bool ok = parse_ini_file(fn, load_handler, &ctx);
+	ok = parse_ini_file(fn, load_handler, &ctx);
 	if (ctx.cur_sect)
 		free(ctx.cur_sect);
+	if (ok && !ctx.got_main_sect) {
+		log_error("load_init_file: main section missing from config file");
+		return false;
+	}
 	return ok;
 }
 
@@ -465,7 +511,7 @@ static double parse_time(const char *value)
 	char *endp = NULL;
 
 	errno = 0;
-	v = strtod(value, &endp);
+	v = strtod_dot(value, &endp);
 	if (errno)
 		return -1;
 	if (*endp || endp == value || v < 0) {
@@ -481,7 +527,7 @@ bool cf_set_time_usec(struct CfValue *cv, const char *value)
 	double v = parse_time(value);
 	if (v < 0)
 		return false;
-	*ptr = USEC * v;
+	*ptr = (usec_t)(USEC * v);
 	return true;
 }
 
