@@ -40,7 +40,7 @@ static char *cstr_skip_ws(char *p)
 	return p;
 }
 
-/* parse paramenter name before '=' */
+/* parse parameter name before '=' */
 static char *cstr_get_key(char *p, char **dst_p)
 {
 	char *end;
@@ -104,7 +104,7 @@ static char *cstr_get_value(char *p, char **dst_p)
 }
 
 /*
- * Get key=val pair from connstring.  returns position it stopped
+ * Get key=val pair from connstring.  Returns position it stopped
  * or NULL on error.  EOF is signaled by *key = 0.
  */
 static char * cstr_get_pair(char *p,
@@ -159,16 +159,19 @@ static void set_connect_query(PgDatabase *db, const char *new)
 static void set_autodb(const char *connstr)
 {
 	char *tmp = strdup(connstr);
+	char *old = cf_autodb_connstr;
+
 	if (!tmp) {
 		log_warning("no mem to change autodb_connstr");
 		return;
 	}
-	if (cf_autodb_connstr) {
-		if (strcmp(connstr, cf_autodb_connstr) != 0)
-			tag_autodb_dirty();
-		free(cf_autodb_connstr);
-	}
+
 	cf_autodb_connstr = tmp;
+	if (old) {
+		if (strcmp(connstr, old) != 0)
+			tag_autodb_dirty();
+		free(old);
+	}
 }
 
 /* fill PgDatabase from connstr */
@@ -177,9 +180,12 @@ bool parse_database(void *base, const char *name, const char *connstr)
 	char *p, *key, *val;
 	PktBuf *msg;
 	PgDatabase *db;
+	struct CfValue cv;
 	int pool_size = -1;
 	int res_pool_size = -1;
+	int max_db_connections = -1;
 	int dbname_ofs;
+	int pool_mode = POOL_INHERIT;
 
 	char *tmp_connstr;
 	const char *dbname = name;
@@ -187,6 +193,7 @@ bool parse_database(void *base, const char *name, const char *connstr)
 	char *port = "5432";
 	char *username = NULL;
 	char *password = "";
+	char *auth_username = NULL;
 	char *client_encoding = NULL;
 	char *datestyle = NULL;
 	char *timezone = NULL;
@@ -195,6 +202,9 @@ bool parse_database(void *base, const char *name, const char *connstr)
 
 	int v_port;
 
+	cv.value_p = &pool_mode;
+	cv.extra = (const void *)pool_mode_map;
+
 	if (strcmp(name, "*") == 0) {
 		set_autodb(connstr);
 		return true;
@@ -210,34 +220,45 @@ bool parse_database(void *base, const char *name, const char *connstr)
 		if (p == NULL) {
 			log_error("%s: syntax error in connstring", name);
 			goto fail;
-		} else if (!key[0])
+		} else if (!key[0]) {
 			break;
+		}
 
-		if (strcmp("dbname", key) == 0)
+		if (strcmp("dbname", key) == 0) {
 			dbname = val;
-		else if (strcmp("host", key) == 0)
+		} else if (strcmp("host", key) == 0) {
 			host = val;
-		else if (strcmp("port", key) == 0)
+		} else if (strcmp("port", key) == 0) {
 			port = val;
-		else if (strcmp("user", key) == 0)
+		} else if (strcmp("user", key) == 0) {
 			username = val;
-		else if (strcmp("password", key) == 0)
+		} else if (strcmp("password", key) == 0) {
 			password = val;
-		else if (strcmp("client_encoding", key) == 0)
+		} else if (strcmp("auth_user", key) == 0) {
+			auth_username = val;
+		} else if (strcmp("client_encoding", key) == 0) {
 			client_encoding = val;
-		else if (strcmp("datestyle", key) == 0)
+		} else if (strcmp("datestyle", key) == 0) {
 			datestyle = val;
-		else if (strcmp("timezone", key) == 0)
+		} else if (strcmp("timezone", key) == 0) {
 			timezone = val;
-		else if (strcmp("pool_size", key) == 0)
+		} else if (strcmp("pool_size", key) == 0) {
 			pool_size = atoi(val);
-		else if (strcmp("reserve_pool", key) == 0)
+		} else if (strcmp("reserve_pool", key) == 0) {
 			res_pool_size = atoi(val);
-		else if (strcmp("connect_query", key) == 0)
+		} else if (strcmp("max_db_connections", key) == 0) {
+			max_db_connections = atoi(val);
+		} else if (strcmp("pool_mode", key) == 0) {
+			if (!cf_set_lookup(&cv, val)) {
+				log_error("skipping database %s because"
+					  " of invalid pool mode: %s", name, val);
+				goto fail;
+			}
+		} else if (strcmp("connect_query", key) == 0) {
 			connect_query = val;
-		else if (strcmp("application_name", key) == 0)
+		} else if (strcmp("application_name", key) == 0) {
 			appname = val;
-		else {
+		} else {
 			log_error("skipping database %s because"
 				  " of unknown parameter in connstring: %s", name, key);
 			goto fail;
@@ -276,25 +297,26 @@ bool parse_database(void *base, const char *name, const char *connstr)
 	/* if updating old db, check if anything changed */
 	if (db->dbname) {
 		bool changed = false;
-		if (strcmp(db->dbname, dbname) != 0)
+		if (strcmp(db->dbname, dbname) != 0) {
 			changed = true;
-		else if (!!host != !!db->host)
+		} else if (!!host != !!db->host) {
 			changed = true;
-		else if (host && strcmp(host, db->host) != 0)
+		} else if (host && strcmp(host, db->host) != 0) {
 			changed = true;
-		else if (v_port != db->port)
+		} else if (v_port != db->port) {
 			changed = true;
-		else if (username && !db->forced_user)
+		} else if (username && !db->forced_user) {
 			changed = true;
-		else if (username && strcmp(username, db->forced_user->name) != 0)
+		} else if (username && strcmp(username, db->forced_user->name) != 0) {
 			changed = true;
-		else if (!username && db->forced_user)
+		} else if (!username && db->forced_user) {
 			changed = true;
-		else if ((db->connect_query && !connect_query)
+		} else if ((db->connect_query && !connect_query)
 			 || (!db->connect_query && connect_query)
 			 || (connect_query && strcmp(connect_query, db->connect_query) != 0))
+		{
 			changed = true;
-
+		}
 		if (changed)
 			tag_database_dirty(db);
 	}
@@ -302,6 +324,8 @@ bool parse_database(void *base, const char *name, const char *connstr)
 	/* if pool_size < 0 it will be set later */
 	db->pool_size = pool_size;
 	db->res_pool_size = res_pool_size;
+	db->pool_mode = pool_mode;
+	db->max_db_connections = max_db_connections;
 
 	if (db->host)
 		free(db->host);
@@ -345,13 +369,23 @@ bool parse_database(void *base, const char *name, const char *connstr)
 		pktbuf_put_string(msg, appname);
 	}
 
-	/* if user is forces, create fake object for it */
+	if (auth_username != NULL) {
+		db->auth_user = find_user(auth_username);
+		if (!db->auth_user) {
+			db->auth_user = add_user(auth_username, "");
+		}
+	} else if (db->auth_user) {
+		db->auth_user = NULL;
+	}
+
+	/* if user is forced, create fake object for it */
 	if (username != NULL) {
 		if (!force_user(db, username, password))
 			log_warning("db setup failed, trying to continue");
-	} else if (db->forced_user)
+	} else if (db->forced_user) {
 		log_warning("losing forced user not supported,"
 			    " keeping old setting");
+	}
 
 	/* remember dbname */
 	db->dbname = (char *)msg->buf + dbname_ofs;
@@ -362,6 +396,64 @@ fail:
 	return true;
 }
 
+bool parse_user(void *base, const char *name, const char *connstr)
+{
+	char *p, *key, *val, *tmp_connstr;
+	PgUser *user;
+	struct CfValue cv;
+	int pool_mode = POOL_INHERIT;
+	int max_user_connections = -1;
+
+
+	cv.value_p = &pool_mode;
+	cv.extra = (const void *)pool_mode_map;
+
+	tmp_connstr = strdup(connstr);
+	if (!tmp_connstr)
+		return false;
+
+	p = tmp_connstr;
+	while (*p) {
+		p = cstr_get_pair(p, &key, &val);
+		if (p == NULL) {
+			log_error("%s: syntax error in user settings", name);
+			goto fail;
+		} else if (!key[0]) {
+			break;
+		}
+
+		if (strcmp("pool_mode", key) == 0) {
+			if (!cf_set_lookup(&cv, val)) {
+				log_error("skipping user %s because"
+					  " of invalid pool mode: %s", name, val);
+				goto fail;
+			}
+		} else if (strcmp("max_user_connections", key) == 0) {
+			max_user_connections = atoi(val);
+		} else {
+			log_error("skipping user %s because"
+				  " of unknown parameter in settings: %s", name, key);
+			goto fail;
+		}
+	}
+
+	user = find_user(name);
+	if (!user) {
+		user = add_user(name, "");
+		if (!user) {
+			log_error("cannot create user, no memory?");
+			goto fail;
+		}
+	}
+
+	user->pool_mode = pool_mode;
+	user->max_user_connections = max_user_connections;
+
+fail:
+	free(tmp_connstr);
+	return true;
+}
+
 /*
  * User file parsing
  */
@@ -463,6 +555,7 @@ bool load_auth_file(const char *fn)
 		return false;
 	}
 
+	log_debug("loading auth_file: \"%s\"", fn);
 	disable_users();
 
 	p = buf;
@@ -472,6 +565,12 @@ bool load_auth_file(const char *fn)
 		if (!*p)
 			break;
 
+		/* skip commented-out lines */
+		if (*p == ';') {
+			while (*p && *p != '\n') p++;
+			continue;
+		}
+
 		/* start of line */
 		if (*p != '"') {
 			log_error("broken auth file");
