@@ -73,10 +73,10 @@ void pktbuf_static(PktBuf *buf, uint8_t *data, int len)
 	buf->fixed_buf = 1;
 }
 
+static PktBuf *temp_pktbuf;
+
 struct PktBuf *pktbuf_temp(void)
 {
-	static PktBuf *temp_pktbuf;
-
 	if (!temp_pktbuf)
 		temp_pktbuf = pktbuf_dynamic(512);
 	if (!temp_pktbuf)
@@ -85,6 +85,12 @@ struct PktBuf *pktbuf_temp(void)
 	return temp_pktbuf;
 }
 
+void pktbuf_cleanup(void)
+{
+	pktbuf_free(temp_pktbuf);
+	temp_pktbuf = NULL;
+}
+
 bool pktbuf_send_immediate(PktBuf *buf, PgSocket *sk)
 {
 	uint8_t *pos = buf->buf + buf->send_pos;
