@@ -85,7 +85,7 @@ static void cleanup_sockets(void)
 static bool add_listen(int af, const struct sockaddr *sa, int salen)
 {
 	struct ListenSocket *ls;
-	int sock, res, val;
+	int sock, res;
 	char buf[128];
 	const char *errpos;
 
@@ -97,14 +97,28 @@ static bool add_listen(int af, const struct sockaddr *sa, int salen)
 	if (sock < 0)
 		goto failed;
 
+	/* SO_REUSEADDR behaviour it default in WIN32.  */
+#ifndef WIN32
 	/* relaxed binding */
 	if (af != AF_UNIX) {
-		val = 1;
+		int val = 1;
 		errpos = "setsockopt";
 		res = setsockopt(sock, SOL_SOCKET, SO_REUSEADDR, &val, sizeof(val));
 		if (res < 0)
 			goto failed;
 	}
+#endif
+
+#ifdef IPV6_V6ONLY
+	/* avoid ipv6 socket's attempt to takeover ipv4 port */
+	if (af == AF_INET6) {
+		int val = 1;
+		errpos = "setsockopt/IPV6_V6ONLY";
+		res = setsockopt(sock, IPPROTO_IPV6, IPV6_V6ONLY, &val, sizeof(val));
+		if (res < 0)
+			goto failed;
+	}
+#endif
 
 	/* bind it */
 	errpos = "bind";
@@ -188,14 +202,14 @@ static void create_unix_socket(const char *socket_dir, int listen_port)
  *
  * Related to tcp_synack_retries sysctl, default 5 (corresponds 180 secs).
  *
- * SO_ACCEPTFILTER needs to be set after listern(), maybe TCP_DEFER_ACCEPT too.
+ * SO_ACCEPTFILTER needs to be set after listen(), maybe TCP_DEFER_ACCEPT too.
  */
 static void tune_accept(int sock, bool on)
 {
 	const char *act = on ? "install" : "uninstall";
 	int res = 0;
 #ifdef TCP_DEFER_ACCEPT
-	int val = 45; /* fixme: proper value */
+	int val = 45; /* FIXME: proper value */
 	socklen_t vlen = sizeof(val);
 	res = getsockopt(sock, IPPROTO_TCP, TCP_DEFER_ACCEPT, &val, &vlen);
 	log_noise("old TCP_DEFER_ACCEPT on %d = %d", sock, val);
@@ -253,10 +267,11 @@ static const char *addrpair(const PgAddr *src, const PgAddr *dst)
 
 static const char *conninfo(const PgSocket *sk)
 {
-	if (is_server_socket(sk))
+	if (is_server_socket(sk)) {
 		return addrpair(&sk->local_addr, &sk->remote_addr);
-	else
+	} else {
 		return addrpair(&sk->remote_addr, &sk->local_addr);
+	}
 }
 
 /* got new connection, associate it with client struct */
@@ -300,14 +315,6 @@ loop:
 
 	log_noise("new fd from accept=%d", fd);
 	if (is_unix) {
-		uid_t uid;
-		gid_t gid;
-		log_noise("getuid(): %d", (int)getuid());
-		if (getpeereid(fd, &uid, &gid) >= 0)
-			log_noise("unix peer uid: %d", (int)uid);
-		else
-			log_warning("unix peer uid failed: %s", strerror(errno));
-
 		client = accept_client(fd, true);
 	} else {
 		client = accept_client(fd, false);
@@ -333,6 +340,8 @@ bool use_pooler_socket(int sock, bool is_unix)
 		return false;
 
 	ls = calloc(1, sizeof(*ls));
+	if (!ls)
+		return false;
 	ls->fd = sock;
 	if (is_unix) {
 		pga_set(&ls->addr, AF_UNIX, cf_listen_port);
